export const extractDirectedRelation = (config, direction)=>{
    const split = {
        here: config[`vertex${direction == 'A->B' ? 'A' : 'B'}`].anchor.split('.'),
        there: config[`vertex${direction == 'A->B' ? 'B' : 'A'}`].anchor.split('.')
    };
    return {
        here: {
            collection: split.here[0],
            list: split.here[1],
            field: split.here[2]
        },
        there: {
            collection: split.there[0],
            list: split.there[1],
            field: split.there[2]
        }
    };
};
export const getId = (entry)=>typeof entry == 'string' ? entry : entry ? entry.id : undefined;
export const getList = (config, direction, data)=>data[extractDirectedRelation(config, direction).here.list];
const getRelationMeta = (entry, field)=>{
    const relationMeta = Object.assign({}, entry);
    delete relationMeta[field];
    delete relationMeta['id'];
    return relationMeta;
};
export const couple = async (docHere, relationEntries, config, direction, req, mode, context)=>{
    const { here, there } = extractDirectedRelation(config, direction);
    const docHereId = getId(docHere);
    if (docHereId) for (const entryHere of relationEntries){
        const docThereId = getId(entryHere[here.field]);
        if (docThereId) {
            const relationMetaHere = getRelationMeta(entryHere, here.field);
            const newEntryThere = {
                [there.field]: docHereId,
                ...relationMetaHere
            };
            try {
                const entriesThere = (await req.payload.findByID({
                    collection: there.collection,
                    id: docThereId,
                    depth: 2
                }))[there.list];
                const matches = entriesThere.filter((entryThere)=>getId(entryThere[there.field]) == docHereId);
                let updateNeeded = mode == 'decoupling' ? matches.length > 0 : matches.length != 1 ? true : !deepComparison(relationMetaHere, getRelationMeta(matches[0], there.field));
                const depth = context.depth ?? 0;
                if (updateNeeded && depth < 1) await req.payload.update({
                    collection: there.collection,
                    id: docThereId,
                    data: {
                        [there.list]: [
                            ...entriesThere.filter((entry)=>getId(entry[there.field]) != docHereId).map((entry)=>{
                                entry[there.field] = getId(entry[there.field]);
                                return entry;
                            }),
                            ...mode == 'decoupling' ? [] : [
                                newEntryThere
                            ]
                        ]
                    },
                    context: {
                        ...context,
                        depth: depth + 1
                    }
                });
            } catch (e) {
                console.log(e);
            }
        }
    }
};
export const deepComparison = (object1, object2)=>{
    const keys1 = Object.keys(object1);
    const keys2 = Object.keys(object2);
    if (keys1.length !== keys2.length) return false;
    if (keys1.filter((key)=>!keys2.includes(key)).length > 0) return false;
    for (const key of keys1){
        const val1 = object1[key];
        const val2 = object2[key];
        const deepen = val1 != null && typeof val1 === 'object' && val2 != null && typeof val2 === 'object';
        if (deepen && !deepComparison(val1, val2) || !deepen && val1 !== val2) return false;
    }
    return true;
};

//# sourceMappingURL=helpers.js.map